(** Routines to read and operate upon files pertaining to secondary
    structure *)

(** General problem with secondary structure *)
exception Bad_ss

(** Read secondary structure from a FASTA style file, single line *)
let read_fasta_ss fname =
  let f = open_in fname in
  let _ = input_line f in
  let ss = input_line f in
  close_in f;
  ss;;

(** Types of secondary structure *)
type ss_type = Coil | Helix | Strand;;
(** Convert characters into secondary structure types *)
let ss_type c = match c with 'H'->Helix | 'E'->Strand | _->Coil;;
(** Types to long name strings *)
let ss_longname t = match t with Coil->"Coil" | Helix->"Helix" | Strand->"Strand";;
(** Types to short name strings *)
let ss_shortname t = match t with Coil->"C" | Helix->"H" | Strand->"S";;
(** Map of ss to numeric vector index *)
let ss2idx = function
    Coil	-> 0
  | Helix	-> 1
  | Strand	-> 2;;

(** Type for SS elements *)
type sselem = (ss_type * int * int * int) list;;

(** No secondary structure *)
let no_ss : sselem = [];;

(** Create a list of secondary structure elements:
    (type,length,start,stop), optional argument [len] is the minimum
    length of the element. *)
let make_ss_elems ?(len=0) sstr =
  let n = String.length sstr in
  let rec next_elem start =
    if start >= n then []
    else begin
      let c = ss_type sstr.[start] in
      let i = ref (start+1) in
      while !i<n && ss_type sstr.[!i] = c do incr i; done;
      if !i-start >= len then
	let elem =  (c, (!i-start), start, (!i-1)) in
	elem::(next_elem !i)
      else
	next_elem !i
    end
  in
  next_elem 0;;

(** Filter only the helix and strand SS types *)
let helix_strand_elems ch =
  let add c lst =
    match c with
      (Helix,_,_,_) | (Strand,_,_,_) -> c::lst
    | (Coil,_,_,_) -> lst in
  List.fold_right add ch [];;

(** If residue [idx] is in a secondary structure element, return
    [Some(idx,jdx,sstype)] with [idx] as the numeric index of the secondary
    structure element (0th helix, 1st sheet, etc) and [jdx] as the
    index in that element (e.g. 0th residue in helix). [sstype] is the
    type of secondary structure.

    If residue is not in a SS element, return [None]
*)
let get_elem ss idx =
  let rec f elems i =
    match elems with
      [] -> None
    | (sstype,_,start,stop)::tl ->
	if start<=idx && idx<=stop then
	  Some(i,idx-start,sstype)
	else f tl (i+1)
  in f ss 0;;

(** Determine whether residue [idx] and residue [jdx] are in the same
    secondary structure element. *)
let same_elem ss idx jdx =
  let ssi = get_elem ss idx in
  let ssj = get_elem ss jdx in
  match ssi,ssj with
    Some(i,_,_),Some(j,_,_) -> i=j
  | _ -> false;;



(*   let filt ch lst = match ch with *)
(*       _,l,_,_ when l>=len -> ch::lst *)
(*     | _ -> lst in *)
(*   List.fold_right filt [] chunks;; *)

      


(* type dssp = { *)
(*   num :int array ;  *)
(*   res :int array ;  *)
(*   chain :string ;  *)
(*   aa :string ;  *)
(*   struct1 :string ;  *)
(*   struct2 :string ;  *)
(*   bp1 :int array ;  *)
(*   bp2 :int array ;  *)
(*   acc :int array ;  *)
(*   nho :string array ;  *)
(*   ohn :string array ;  *)
(*   nho2 :string array ;  *)
(*   ohn2 :string array ;  *)
(*   tco :float array ;  *)
(*   kappa :float array ;  *)
(*   alpha :float array ;  *)
(*   phi :float array ;  *)
(*   psi :float array ;  *)
(*   x :float array ;  *)
(*   y :float array ;  *)
(*   z :float array;  *)
(* };; *)
(*
#  #  RESIDUE AA STRUCTURE BP1 BP2  ACC     N-H-->O    O-->H-N    N-H-->O    O-->H-N    TCO  KAPPA ALPHA  PHI   PSI    X-CA   Y-CA   Z-CA 
#    6    6 A L  T 3<5S-     0   0   22     -4,-1.3    -1,-0.3    -5,-0.1    -2,-0.2   0.643 125.6 -97.9 -80.7 -14.7   22.2   23.8   -5.6
# >  | >  | | >| | >     | > | > | >  | > | >     | >| >     | >| >     | >| >     | |>    |>    |>    |>    |>    |>     |>     |>     |   
# 
#	 4	 num		 $2	 
#	 4	 res		 $4	 
#	 1	 chain		 $6	 
#	 2	 aa		 $8	 
#	 1	 struct		 $10	 
#	 7	 struct2	 $12	 
#	 3	 bp1		 $14	 
#	 3	 bp2		 $16	 
#        4       acc             $18
#	 3	 blank		 $20
#	 7	 nho		 $22
#	 2	 blank		 $24
#	 7	 ohn		 $26
#	 2	 blank		 $28
#	 7	 nho2		 $30
#	 2	 blank		 $32
#	 7	 ohn2		 $34
#	 2	 blank		 $35
#	 6	 tco		 $36
#	 6	 kappa		 $37
#	 6	 alpha		 $38
#	 6	 phi		 $39
#	 6	 psi		 $40
#	 7	 x		 $41
#	 7	 y		 $42
#	 7	 z		 $43
*)
(* let parse_record r = *)
(*   let rec_formati = *)
(*     "%5[^~]s%5[^~]s%1[^~]s%1[^~]s%2[^~]s%3[^~]s%8[^~]s%4[^~]s%5[^~]s%4[^~]s%1[^~]s%[^~]s%1[^~]s%1[^~]s%[^~]s%1[^~]s%8[^~]s%6[^~]s%6[^~]s%6[^~]s%6[^~]s%7[^~]s%7[^~]s%7[^~]s" *)
(*   in *)
(*   let  *)

  
